@@ -3,10 +3,11 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Generate walking basslines
 
-our $VERSION = '0.0312';
+our $VERSION = '0.0400';
 
 use Data::Dumper::Compact qw(ddc);
 use Carp qw(croak);
+use List::MoreUtils qw(first_index);
 use List::Util qw(any min uniq);
 use Music::Chord::Note;
 use Music::Note;
@@ -27,6 +28,20 @@ has guitar => (
 );
 
 
+has modal => (
+    is      => 'ro',
+    isa     => sub { croak 'not a boolean' unless $_[0] =~ /^[01]$/ },
+    default => sub { 0 },
+);
+
+
+has keycenter => (
+    is      => 'ro',
+    isa     => sub { croak 'not a valid key' unless $_[0] =~ /^[A-G][#b]?$/ },
+    default => sub { 'C' },
+);
+
+
 has intervals => (
     is      => 'ro',
     isa     => sub { croak 'not an array reference' unless ref $_[0] eq 'ARRAY' },
@@ -71,9 +86,6 @@ sub generate {
     print "CHORD: $chord\n" if $self->verbose;
     print "NEXT: $next_chord\n" if $self->verbose && $next_chord;
 
-    my $scale = $self->scale->($chord);
-    my $next_scale = defined $next_chord ? $self->scale->($next_chord) : '';
-
     # Parse the chord
     my $chord_note;
     my $flavor;
@@ -88,6 +100,23 @@ sub generate {
         $next_chord_note = $1;
     }
 
+    my ($scale, $next_scale);
+    if ($self->modal) {
+        print "MODAL\n" if $self->verbose;
+        my @modes = qw( ionian dorian phrygian lydian mixolydian aeolian locrian );
+        my @key_notes = get_scale_notes($self->keycenter, $modes[0]);
+        my $position = first_index { $_ eq $chord_note } @key_notes;
+        $scale = $position >= 0 ? $modes[$position] : $modes[0];
+        if ($next_chord_note) {
+            $position = first_index { $_ eq $next_chord_note } @key_notes;
+            $next_scale = $position >= 0 ? $modes[$position] : $modes[0];
+        }
+    }
+    else {
+        $scale = $self->scale->($chord);
+        $next_scale = defined $next_chord ? $self->scale->($next_chord) : '';
+    }
+
     my $cn = Music::Chord::Note->new;
 
     my @notes = map { $self->pitchnum($_) }
@@ -123,7 +152,7 @@ sub generate {
             $n->en_eq('sharp');
         }
         my $y = $n->format('isobase');
-        if (($scale eq 'major' || $scale eq 'minor')
+        if (($scale eq 'major' || $scale eq 'ionian' || $scale eq 'minor' || $scale eq 'aeolian')
             && (
             ($flavor =~ /[#b]5/ && ($x eq $tones[4] || $y eq $tones[4]))
             ||
@@ -233,7 +262,7 @@ MIDI::Bassline::Walk - Generate walking basslines
 
 =head1 VERSION
 
-version 0.0312
+version 0.0400
 
 =head1 SYNOPSIS
 
@@ -241,8 +270,13 @@ version 0.0312
 
   my $bassline = MIDI::Bassline::Walk->new(verbose => 1);
 
-  my $notes = $bassline->generate('C7b5', 8);
+  $bassline = MIDI::Bassline::Walk->new(
+    guitar    => 1,
+    modal     => 1,
+    keycenter => 'Bb',
+  );
 
+  my $notes = $bassline->generate('F7b5', 8);
   # MIDI:
   # $score->n('qn', $_) for @$notes;
 
@@ -272,9 +306,25 @@ Transpose notes below C<E2> (C<40>) up an octave.
 
 Default: C<0>
 
+=head2 modal
+
+  $modal = $bassline->modal;
+
+Maintain the key-center and only choose notes within a mode.
+
+Default: C<0>
+
+=head2 keycenter
+
+  $keycenter = $bassline->keycenter;
+
+The key-center for B<modal> accompaniment.
+
+Default: C<C>
+
 =head2 intervals
 
-  $verbose = $bassline->intervals;
+  $intervals = $bassline->intervals;
 
 Allowed intervals passed to L<Music::VoiceGen>.
 
@@ -290,7 +340,7 @@ Default: C<2>
 
 =head2 scale
 
-  $scale = $bassline->scale;
+  $scale = $bassline->scale->($chord);
 
 The musical scale to use, based on a given chord (i.e. C<$_[0]> here).
 
@@ -336,6 +386,8 @@ Default: C<0>
       intervals => $intervals,
       octave    => $octave,
       scale     => $scale,
+      modal     => $modal,
+      keycenter => $key_center,
       verbose   => $verbose,
   );
 
@@ -354,6 +406,11 @@ If given a B<next_chord>, perform an intersection of the two scales,
 and replace the final note of the generated phrase with a note of the
 intersection, if there are notes in common.
 
+If the B<modal> attribute is set, then the chosen notes will be within
+the modal scale given the B<keycenter> setting.  If it is not set (the
+default), notes will be chosen as if the key has changed to the
+current chord!
+
 Defaults:
 
   chord: C
